package org.apache.bcel.generic;

import java.io.DataOutputStream;
import java.io.IOException;
import org.apache.bcel.Const;
import org.apache.bcel.classfile.Constant;
import org.apache.bcel.classfile.ConstantClass;
import org.apache.bcel.classfile.ConstantPool;
import org.apache.bcel.util.ByteSequence;

public abstract class CPInstruction extends Instruction implements TypedInstruction, IndexedInstruction {
   /** @deprecated */
   @Deprecated
   protected int index;

   CPInstruction() {
   }

   protected CPInstruction(short opcode, int index) {
      super(opcode, (short)3);
      this.setIndex(index);
   }

   public void dump(DataOutputStream out) throws IOException {
      out.writeByte(super.getOpcode());
      out.writeShort(this.index);
   }

   public final int getIndex() {
      return this.index;
   }

   public Type getType(ConstantPoolGen cpg) {
      ConstantPool cp = cpg.getConstantPool();
      String name = cp.getConstantString(this.index, (byte)7);
      if (!name.startsWith("[")) {
         name = "L" + name + ";";
      }

      return Type.getType(name);
   }

   protected void initFromFile(ByteSequence bytes, boolean wide) throws IOException {
      this.setIndex(bytes.readUnsignedShort());
      super.setLength(3);
   }

   public void setIndex(int index) {
      if (index < 0) {
         throw new ClassGenException("Negative index value: " + index);
      } else {
         this.index = index;
      }
   }

   public String toString(boolean verbose) {
      return super.toString(verbose) + " " + this.index;
   }

   public String toString(ConstantPool cp) {
      Constant c = cp.getConstant(this.index);
      String str = cp.constantToString(c);
      if (c instanceof ConstantClass) {
         str = str.replace('.', '/');
      }

      return Const.getOpcodeName(super.getOpcode()) + " " + str;
   }
}
